#include "../config/pathan_config.h"
/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include <assert.h>

#include <pathan/internal/factory/DatatypeFactory.hpp>
#include <pathan/DynamicContext.hpp>
#include <pathan/functions/FunctionConstructor.hpp> // for datatype-uri
#include "../items/impl/ATBooleanOrDerivedImpl.hpp"
#include <pathan/internal/items/ATDecimalOrDerivedImpl.hpp>
#include <pathan/internal/items/ATDoubleOrDerivedImpl.hpp>
#include "../items/impl/ATFloatOrDerivedImpl.hpp"
#include "../items/impl/ATQNameOrDerivedImpl.hpp"
#include <pathan/ATDurationOrDerived.hpp>
#include <pathan/ATUntypedAtomic.hpp>

#include <xercesc/util/XMLString.hpp>
#include <xercesc/validators/schema/SchemaSymbols.hpp>

DatatypeFactory::DatatypeFactory()
{
}

DatatypeFactory::~DatatypeFactory()
{
}

/////////////////////////////
// Helper creation methods //
/////////////////////////////

const ATQNameOrDerived* DatatypeFactory::STR2AT::createQName(XPath2MemoryManager *xm, const XMLCh* uri,
                                                         const XMLCh* name, 
                                                         const DynamicContext* context
                                                          ) {
  return STR2AT::createQNameOrDerived(
	  xm,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_QNAME,
      uri,
      name,
      context
      );
}

const ATDoubleOrDerived* DatatypeFactory::POD2AT::createDouble(XPath2MemoryManager *xm, const MAPM value, const DynamicContext* context) {
  return POD2AT::createDoubleOrDerived(
	  xm,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DOUBLE,
      value, context);
}

const ATDoubleOrDerived* DatatypeFactory::STR2AT::createDouble(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context) {
  return STR2AT::createDoubleOrDerived(
	  xm,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DOUBLE,
      value, context);
}

const ATFloatOrDerived* DatatypeFactory::POD2AT::createFloat(XPath2MemoryManager *xm, const MAPM value, const DynamicContext* context) {
  return POD2AT::createFloatOrDerived(
	  xm,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_FLOAT,
      value, context);
}

const ATFloatOrDerived* DatatypeFactory::STR2AT::createFloat(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context) {
  return STR2AT::createFloatOrDerived(
	  xm,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_FLOAT,
      value, context);
}

const ATDecimalOrDerived* DatatypeFactory::POD2AT::createDecimal(XPath2MemoryManager *xm, const MAPM value, const DynamicContext* context) {
  return POD2AT::createDecimalOrDerived(
	  xm,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DECIMAL,
      value, context);
}

const ATDecimalOrDerived* DatatypeFactory::STR2AT::createDecimal(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context) {
  return STR2AT::createDecimalOrDerived(
	  xm,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DECIMAL,
      value, context);
}

const ATDecimalOrDerived* DatatypeFactory::POD2AT::createInteger(XPath2MemoryManager *xm, const MAPM value, const DynamicContext* context) {
  return POD2AT::createDecimalOrDerived(
	  xm,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_INTEGER,
      value, context);
}

const ATDecimalOrDerived* DatatypeFactory::STR2AT::createInteger(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context) {
  return STR2AT::createDecimalOrDerived(
	  xm,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_INTEGER,
      value, context);
}

const ATBooleanOrDerived* DatatypeFactory::POD2AT::createBoolean(XPath2MemoryManager *xm, bool value, const DynamicContext* context) {
  return POD2AT::createBooleanOrDerived(
	  xm,
	  XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
	  XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_BOOLEAN,
	  value, context);  
}

const ATBooleanOrDerived* DatatypeFactory::STR2AT::createBoolean(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context) {
  return STR2AT::createBooleanOrDerived(
	  xm,
	  XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
	  XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_BOOLEAN,
	  value, context);  
}

const ATDecimalOrDerived* DatatypeFactory::POD2AT::createNonNegativeInteger(XPath2MemoryManager *xm, const MAPM value, const DynamicContext* context) {
  return POD2AT::createDecimalOrDerived(
	  xm,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_NONNEGATIVEINTEGER,
      value, context);
}
      
const ATDurationOrDerived* DatatypeFactory::STR2AT::createDayTimeDuration(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context) {
  return STR2AT::createDurationOrDerived(
	  xm,
      FunctionConstructor::XMLChXPath2DatatypesURI, 
      ATDurationOrDerived::fgDT_DAYTIMEDURATION,
      value, context);
}

const ATDurationOrDerived* DatatypeFactory::STR2AT::createYearMonthDuration(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context) {
  return STR2AT::createDurationOrDerived(
	  xm,
      FunctionConstructor::XMLChXPath2DatatypesURI, 
      ATDurationOrDerived::fgDT_YEARMONTHDURATION,
      value, context);
}

const ATDateOrDerived* DatatypeFactory::STR2AT::createDate(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context) {
  return STR2AT::createDateOrDerived(
	  xm,
	  XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
	  XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DATE,
	  value, context);  
}

const ATDateTimeOrDerived* DatatypeFactory::STR2AT::createDateTime(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context) {
  return STR2AT::createDateTimeOrDerived(
	  xm,
	  XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
	  XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DATETIME,
	  value, context);  
}

const ATTimeOrDerived* DatatypeFactory::STR2AT::createTime(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context) {
  return STR2AT::createTimeOrDerived(
	  xm,
	  XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
	  XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_TIME,
	  value, context);  
}

const ATAnyURIOrDerived* DatatypeFactory::STR2AT::createAnyURI(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context) {
    return STR2AT::createAnyURIOrDerived(
	  xm,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_ANYURI,
      value, context);
}

const ATStringOrDerived* DatatypeFactory::STR2AT::createString(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context) {
  return STR2AT::createStringOrDerived(
	  xm,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_STRING,
      value, context);
}

const ATBase64BinaryOrDerived* DatatypeFactory::STR2AT::createBase64Binary(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context) {
  return STR2AT::createBase64BinaryOrDerived(
	  xm,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_BASE64BINARY,
      value, context);
}

const ATHexBinaryOrDerived* DatatypeFactory::STR2AT::createHexBinary(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context) {
  return STR2AT::createHexBinaryOrDerived(
	  xm,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_HEXBINARY,
      value, context);
}

//////////////////////////
// All creation methods //
//////////////////////////


const AnyAtomicType* DatatypeFactory::STR2AT::createDerivedFromAtomicType(XPath2MemoryManager *xm, const XMLCh* typeURI,
                                        const XMLCh* typeName, 
                                        const XMLCh* value, const DynamicContext* context) {

  const DatatypeFactory* dtf = context->getDatatypeFactory(typeURI, typeName);
  return dtf->createInstance(xm, typeURI, typeName, value, context);
}

const ATAnySimpleType* DatatypeFactory::STR2AT::createAnySimpleType(XPath2MemoryManager *xm, const XMLCh* value, 
                                                  const DynamicContext* context) {
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_ANYSIMPLETYPE);
  return (const ATAnySimpleType*) dtf->createInstance(xm, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, 
                                                      XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_ANYSIMPLETYPE, 
                                                      value, context);

}

/** create a xs:anyURI */
const ATAnyURIOrDerived* DatatypeFactory::STR2AT::createAnyURIOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName, 
                                                  const XMLCh* value, 
                                                  const DynamicContext* context) {

  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_ANYURI);
  return (ATAnyURIOrDerived*) dtf->createInstance(xm, typeURI, typeName, value, context);
}

/** create a xs:base64Binary */
const ATBase64BinaryOrDerived* DatatypeFactory::STR2AT::createBase64BinaryOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value, 
                                                  const DynamicContext* context){
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_BASE64BINARY);
  return (ATBase64BinaryOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);
}

/** create a xs:boolean */
const ATBooleanOrDerived* DatatypeFactory::STR2AT::createBooleanOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value, 
                                                  const DynamicContext* context) {
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_BOOLEAN);
  return (ATBooleanOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);

}

/** create a xs:boolean with a bool value */
const ATBooleanOrDerived* DatatypeFactory::POD2AT::createBooleanOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  bool value, 
                                                  const DynamicContext* context) {

  ATBooleanOrDerivedImpl* tmp =  new (xm) ATBooleanOrDerivedImpl(typeURI, typeName, value, xm, context);

  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_BOOLEAN);    
  if(dtf->checkInstance(typeURI, typeName, tmp->asString(context), context)) {
    return tmp;
  } else {
    // this call will obviously fail, but it is better for error reporting, 
    // since we actually get the XMLException's error message 
    return (ATBooleanOrDerived*)dtf->createInstance(xm, typeURI, typeName, tmp->asString(context), context);
  }
}

/** create a xs:date */
const ATDateOrDerived* DatatypeFactory::STR2AT::createDateOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value, 
                                                  const DynamicContext* context){
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DATE);
  return (ATDateOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);
}


/** create a xs:dateTime */
const ATDateTimeOrDerived* DatatypeFactory::STR2AT::createDateTimeOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value,
                                                  const DynamicContext* context){
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DATETIME);
  return (ATDateTimeOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);
}

/** create a xs:decimal */
const ATDecimalOrDerived* DatatypeFactory::STR2AT::createDecimalOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value,
                                                  const DynamicContext* context){
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DECIMAL);
  return (ATDecimalOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);
}

/** create a xs:decimal with a MAPM */
const ATDecimalOrDerived* DatatypeFactory::POD2AT::createDecimalOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const MAPM value,
                                                  const DynamicContext* context){
  ATDecimalOrDerivedImpl* tmp =  new (xm) ATDecimalOrDerivedImpl(typeURI, typeName, value, xm, context);

    const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DECIMAL);
  if(dtf->checkInstance(typeURI, typeName, tmp->asString(context), context)) {
    return tmp;
  } else {
    // this call will obviously fail, but it is better for error reporting, 
    // since we actually get the XMLException's error message 
    return (ATDecimalOrDerived*)dtf->createInstance(xm, typeURI, typeName, tmp->asString(context), context);
  }
}


/** create a xs:double */  
const ATDoubleOrDerived* DatatypeFactory::STR2AT::createDoubleOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value, 
                                                  const DynamicContext* context){
  if(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::equals(value, Numeric::NAN_string)) {
	value= Numeric::NaN_string;
  }
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DOUBLE);
  return (ATDoubleOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);
}

/** create a xs:double with a MAPM */
const ATDoubleOrDerived* DatatypeFactory::POD2AT::createDoubleOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const MAPM value, 
                                                  const DynamicContext* context){

  ATDoubleOrDerivedImpl* tmp =  new (xm) ATDoubleOrDerivedImpl(typeURI, typeName, value, xm, context);

  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DOUBLE);
  if(dtf->checkInstance(typeURI, typeName, tmp->asString(context), context)) {
    return tmp;
  } else {
    // this call will obviously fail, but it is better for error reporting, 
    // since we actually get the XMLException's error message 
    return (ATDoubleOrDerived*)dtf->createInstance(xm, typeURI, typeName, tmp->asString(context), context);
  }
}


/** create a xs:duration */
const ATDurationOrDerived* DatatypeFactory::STR2AT::createDurationOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value, 
                                                  const DynamicContext* context){
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DURATION);
  return (ATDurationOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);
}

/** create a xs:float */
const ATFloatOrDerived* DatatypeFactory::STR2AT::createFloatOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value, 
                                                  const DynamicContext* context){
  if(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::equals(value, Numeric::NAN_string)) {
	value= Numeric::NaN_string;
  }
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_FLOAT);
  return (ATFloatOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);

}

/** create a xs:float with a MAPM */
const ATFloatOrDerived* DatatypeFactory::POD2AT::createFloatOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const MAPM value, 
                                                  const DynamicContext* context) {
  ATFloatOrDerivedImpl* tmp =  new (xm) ATFloatOrDerivedImpl(typeURI, typeName, value, xm, context);
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_FLOAT);
  if(dtf->checkInstance(typeURI, typeName, tmp->asString(context), context)) {
    return tmp;
  } else {
    // this call will obviously fail, but it is better for error reporting, 
    // since we actually get the XMLException's error message 
    return (ATFloatOrDerived*)dtf->createInstance(xm, typeURI, typeName, tmp->asString(context), context);
  }
}

/** create a xs:gDay */
const ATGDayOrDerived* DatatypeFactory::STR2AT::createGDayOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value, 
                                                  const DynamicContext* context) {
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DAY);
  return (ATGDayOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);
}

/** create a xs:gMonth */
const ATGMonthOrDerived* DatatypeFactory::STR2AT::createGMonthOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value, 
                                                  const DynamicContext* context) {
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_MONTH);
  return (ATGMonthOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);

}


/** create a xs:gMonthDay */
const ATGMonthDayOrDerived* DatatypeFactory::STR2AT::createGMonthDayOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value, 
                                                  const DynamicContext* context) {
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_MONTHDAY);
  return (ATGMonthDayOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);
}

/** create a xs:gYear */
const ATGYearOrDerived* DatatypeFactory::STR2AT::createGYearOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value, 
                                                  const DynamicContext* context) {
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_YEAR);
  return (ATGYearOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);
}


/** create a xs:gYearMonth */
const ATGYearMonthOrDerived* DatatypeFactory::STR2AT::createGYearMonthOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value, 
                                                  const DynamicContext* context) {
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_YEARMONTH);
  return (ATGYearMonthOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);
}

/** create a xs:hexBinary */
const ATHexBinaryOrDerived* DatatypeFactory::STR2AT::createHexBinaryOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value, 
                                                  const DynamicContext* context) {
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_HEXBINARY);
  return (ATHexBinaryOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);
}

/** create a xs:NOTATION*/
const ATNotationOrDerived* DatatypeFactory::STR2AT::createDerivedFromNotation(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value, 
                                                  const DynamicContext* context) {
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER XMLUni::fgNotationString);
  return (ATNotationOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);
}

/** create a xs:QName */
const ATQNameOrDerived* DatatypeFactory::STR2AT::createQNameOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                 const XMLCh* typeName,
                                                 const XMLCh* value, 
                                                 const DynamicContext* context) {
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_QNAME);
  return (ATQNameOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);
}

/** create a xs:QName with two parameters */
const ATQNameOrDerived* DatatypeFactory::STR2AT::createQNameOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                 const XMLCh* typeName,
                                                 const XMLCh* uri,
                                                 const XMLCh* name, 
                                                 const DynamicContext* context) {

  ATQNameOrDerivedImpl* tmp =  new (xm) ATQNameOrDerivedImpl(typeURI, typeName, uri, name, xm, context);
  
  const DatatypeFactory* dtf_anyURI = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_ANYURI);
  if(dtf_anyURI->checkInstance(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_ANYURI, uri, context)) {
    const DatatypeFactory* dtf_NCName = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_STRING);
    if (dtf_NCName->checkInstance(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_NCNAME, name, context)) {
      return tmp;
    } else {
      // this call will obviously fail, but it is better for error reporting, 
      // since we actually get the XMLException's error message 
      return (const ATQNameOrDerived*)dtf_NCName->createInstance(xm, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_NCNAME, name, context);
    }
  } else {
    // this call will obviously fail, but it is better for error reporting, 
    // since we actually get the XMLException's error message 
    return (const ATQNameOrDerived*)dtf_anyURI->createInstance(xm, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_ANYURI, uri, context);
  }
}


/** create a xs:string */
const ATStringOrDerived* DatatypeFactory::STR2AT::createStringOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value, 
                                                  const DynamicContext* context) {
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_STRING);
  assert(context->isTypeOrDerivedFromType(typeURI, typeName, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_STRING));
  return (ATStringOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);
}

/** create a xs:time */
const ATTimeOrDerived* DatatypeFactory::STR2AT::createTimeOrDerived(XPath2MemoryManager *xm, const XMLCh* typeURI, 
                                                  const XMLCh* typeName,
                                                  const XMLCh* value, 
                                                  const DynamicContext* context){
  const DatatypeFactory* dtf = context->getDatatypeFactory(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_TIME);
  return (ATTimeOrDerived*)dtf->createInstance(xm, typeURI, typeName, value, context);
}

/** create an xdt:untypedAtomic */
const ATUntypedAtomic* DatatypeFactory::STR2AT::createUntypedAtomic(XPath2MemoryManager *xm, const XMLCh* value, const DynamicContext* context) {
  const DatatypeFactory* dtf = context->getDatatypeFactory(FunctionConstructor::XMLChXPath2DatatypesURI, ATUntypedAtomic::fgDT_UNTYPEDATOMIC);
  return (ATUntypedAtomic*)dtf->createInstance(xm, FunctionConstructor::XMLChXPath2DatatypesURI, ATUntypedAtomic::fgDT_UNTYPEDATOMIC, value, context);
}



