/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*
  Factory template class for Datatypes
*/

#ifndef _DatatypeFactoryTemplate_HPP
#define _DatatypeFactoryTemplate_HPP



#include <pathan/internal/factory/DatatypeFactory.hpp>

#include <pathan/XPath2MemoryManager.hpp>     // needed for the operator new
#include "../exceptions/InvalidLexicalSpaceException.hpp"
#include <pathan/exceptions/TypeNotFoundException.hpp>
#include <pathan/DocumentCache.hpp>
#include <pathan/DynamicContext.hpp>

#include <xercesc/validators/datatype/DatatypeValidator.hpp>
#include <xercesc/util/XMLException.hpp>
#include <xercesc/framework/XMLBuffer.hpp>

class AnyAtomicType;

///used to facilitate the creation of datatypes
template<class TYPE>
class DatatypeFactoryTemplate : public DatatypeFactory
{
public:

    DatatypeFactoryTemplate(const DocumentCache* dc): fDocumentCache(dc) {  };

    AnyAtomicType* createInstance(XPath2MemoryManager *xm,
                                  const XMLCh* typeURI,
                                  const XMLCh* typeName,
                                  const XMLCh* value,
                                  const DynamicContext* context) const {
        
        // validate before we build the type
        XERCES_CPP_NAMESPACE_QUALIFIER DatatypeValidator* validator = const_cast<XERCES_CPP_NAMESPACE_QUALIFIER DatatypeValidator*>(fDocumentCache->getDatatypeValidator(typeURI, typeName));
        if(validator) {
            try {
                validator->validate(value, 0, context->getMemoryManager());
            } catch (XERCES_CPP_NAMESPACE_QUALIFIER XMLException &e) {
                DSLthrow(InvalidLexicalSpaceException, X("DatatypeFactoryTemplate::createInstance"), e.getMessage());
            }
        }
        else {
            XERCES_CPP_NAMESPACE_QUALIFIER XMLBuffer buf(1023, context->getMemoryManager());
            buf.append(X("Type "));
            buf.append(typeURI);
            buf.append(XERCES_CPP_NAMESPACE_QUALIFIER chColon);
            buf.append(typeName);
            buf.append(X(" not found"));
            DSLthrow(TypeNotFoundException, X("DatatypeFactoryTemplate::createInstance"), buf.getRawBuffer());
        }

        return new (xm) TYPE(typeURI, typeName, value, xm, context);
    };

    bool checkInstance(const XMLCh* typeURI,
                       const XMLCh* typeName,
                       const XMLCh* value,
                       const DynamicContext* context) const {

        XERCES_CPP_NAMESPACE_QUALIFIER DatatypeValidator* validator = const_cast<XERCES_CPP_NAMESPACE_QUALIFIER DatatypeValidator*>(fDocumentCache->getDatatypeValidator(typeURI, typeName));

        if(validator) {
            try {
                validator->validate(value, 0, context->getMemoryManager());
            } catch (XERCES_CPP_NAMESPACE_QUALIFIER XMLException &e) {
                return false;
            }
        }
        else {
            XERCES_CPP_NAMESPACE_QUALIFIER XMLBuffer buf(1023, context->getMemoryManager());
            buf.append(X("Type "));
            buf.append(typeURI);
            buf.append(XERCES_CPP_NAMESPACE_QUALIFIER chColon);
            buf.append(typeName);
            buf.append(X(" not found"));
            DSLthrow(TypeNotFoundException, X("DatatypeFactoryTemplate::createInstance"), buf.getRawBuffer());
        }

        return true;
    };

    AnyAtomicType::AtomicObjectType getPrimitiveTypeIndex() const
    {
        return TYPE::getTypeIndex();
    };

    const XMLCh* getPrimitiveTypeName() const {
        return TYPE::getPrimitiveName();
    };

private:
    const DocumentCache *fDocumentCache;

};

#endif // _DatatypeFactoryTemplate_HPP
