#include "../config/pathan_config.h"
/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include <pathan/functions/FunctionInsertBefore.hpp>
#include <pathan/Sequence.hpp>
#include <pathan/Node.hpp>
#include <pathan/ATDecimalOrDerived.hpp>
#include <pathan/DynamicContext.hpp>
#include <pathan/internal/factory/DatatypeFactory.hpp>

const XMLCh FunctionInsertBefore::name[] = { XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chDash, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_b, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_f, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chNull };

/**
 * fn:insert-before($target as item()*, $position as xs:integer, $inserts as item()*) as item()*
 */

FunctionInsertBefore::FunctionInsertBefore(const VectorOfDataItems &args, XPath2MemoryManager* memMgr)
  : ConstantFoldingFunction(name,3, 3, "item()*, integer, item()*", args, memMgr)
{
}

Sequence FunctionInsertBefore::collapseTreeInternal(DynamicContext* context, int flags) const
{
  XPath2MemoryManager* memMgr = context->getMemoryManager();

	Sequence origSeq = getParamNumber(1, context);
  Sequence insertSeq = getParamNumber(3, context);
  
  if(origSeq.isEmpty())
    return insertSeq;
  if(insertSeq.isEmpty())
    return origSeq;
    
  const ATDecimalOrDerived* pos = ((const ATDecimalOrDerived*)getParamNumber(2, context).first());
  const ATDecimalOrDerived* origSeqLen = DatatypeFactory::POD2AT::createInteger(memMgr, (long)origSeq.getLength(), context);
  
  Sequence retSeq = Sequence(origSeq.getLength()+insertSeq.getLength(),memMgr);
  
  if(!(pos->lessThan(origSeqLen, context))) {
		retSeq.joinSequence(origSeq);
    retSeq.joinSequence(insertSeq);
    return retSeq;
  }
  
  const ATDecimalOrDerived* one = DatatypeFactory::POD2AT::createInteger(memMgr, 1, context);
  
  if(pos->lessThan(one, context)) {
		retSeq.joinSequence(insertSeq);
    retSeq.joinSequence(origSeq);
    return retSeq;
  }

  const ATDecimalOrDerived* i = DatatypeFactory::POD2AT::createInteger(memMgr, 0, context);
  for(; i->lessThan(pos, context); i = (const ATDecimalOrDerived*)i->add(one, context)) {
    retSeq.addItem(origSeq.item(i));
  }

  retSeq.joinSequence(insertSeq);

  for(; i->lessThan(origSeqLen, context); i = (const ATDecimalOrDerived*)i->add(one, context)) {
    retSeq.addItem(origSeq.item(i));
  }

	return retSeq;
}












