#include "../config/pathan_config.h"
/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include <pathan/functions/FunctionRemove.hpp>
#include <pathan/dataItem/DataItemLiteral.hpp>
#include <pathan/Sequence.hpp>
#include <pathan/Node.hpp>
#include <pathan/ATStringOrDerived.hpp>
#include <pathan/ATDoubleOrDerived.hpp>
#include <pathan/ATDecimalOrDerived.hpp>
#include <pathan/DynamicContext.hpp>
#include <pathan/internal/factory/DatatypeFactory.hpp>

const XMLCh FunctionRemove::name[] = {
XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_m, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_v, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chNull };

/**
 * fn:remove($target as item()*, $position as xs:integer) as item()*
 */

FunctionRemove::FunctionRemove(const VectorOfDataItems &args, XPath2MemoryManager* memMgr)
  : ConstantFoldingFunction(name,2, 2, "item()*, integer", args, memMgr)
{
}

Sequence FunctionRemove::collapseTreeInternal(DynamicContext* context, int flags) const
{
	XPath2MemoryManager* memMgr = context->getMemoryManager();

	Sequence origSeq = getParamNumber(1, context);
	const ATDecimalOrDerived* pos = (const ATDecimalOrDerived*)getParamNumber(2, context).first();
  const ATDecimalOrDerived* one = DatatypeFactory::POD2AT::createInteger(memMgr, 1, context);

  const ATDecimalOrDerived* floorPos = (const ATDecimalOrDerived*)pos->floor(context)->subtract(one, context);
  const ATDecimalOrDerived* origSeqLen = DatatypeFactory::POD2AT::createInteger(memMgr, (long)origSeq.getLength(), context);
  
  Sequence removeSeq = Sequence(origSeq.getLength(),memMgr);
  
  const ATDecimalOrDerived* i = DatatypeFactory::POD2AT::createInteger(memMgr, 0, context);
  for(; !i->equals(origSeqLen, context); i = (const ATDecimalOrDerived*)i->add(one, context)) {
    if(!i->equals(floorPos, context)) {
      removeSeq.addItem(origSeq.item(i));
    }
  }
  
	return removeSeq;
}
