#include "../../config/pathan_config.h"
/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include <pathan/functions/FunctionSubtractDatesYieldingYearMonthDuration.hpp>
#include "../../utils/DateUtils.hpp"
#include <pathan/DynamicContext.hpp>
#include <pathan/ATDateOrDerived.hpp>
#include <pathan/ATDateTimeOrDerived.hpp>
#include <pathan/ATDurationOrDerived.hpp>
#include <xercesc/validators/schema/SchemaSymbols.hpp>

const XMLCh FunctionSubtractDatesYieldingYearMonthDuration::name[] = {
XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_u, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_b, 
XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a, 
XERCES_CPP_NAMESPACE_QUALIFIER chLatin_c, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chDash, 
XERCES_CPP_NAMESPACE_QUALIFIER chLatin_d, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, 
XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s, XERCES_CPP_NAMESPACE_QUALIFIER chDash, 
XERCES_CPP_NAMESPACE_QUALIFIER chLatin_y, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, 
XERCES_CPP_NAMESPACE_QUALIFIER chLatin_l, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_d, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, 
XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_g, XERCES_CPP_NAMESPACE_QUALIFIER chDash, 
XERCES_CPP_NAMESPACE_QUALIFIER chLatin_y, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a, 
XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_M, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, 
XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_h, 
XERCES_CPP_NAMESPACE_QUALIFIER chLatin_D, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_u, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, 
XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, 
XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chNull };


/*
 * fn:subtract-dates-yielding-yearMonthDuration($arg1 as xs:date?, $arg2 as xs:date?) as xdt:yearMonthDuration?
 */

FunctionSubtractDatesYieldingYearMonthDuration::FunctionSubtractDatesYieldingYearMonthDuration(const VectorOfDataItems &args, XPath2MemoryManager* memMgr)
  : DataItemFunction(name,2, 2, "date?, date?", args, memMgr)
{
}

DataItem* FunctionSubtractDatesYieldingYearMonthDuration::staticResolution(StaticContext *context, StaticResolutionContext *src)
{
  return resolveDataItemsForDateOrTime(_args, context, src, true);
}

Sequence FunctionSubtractDatesYieldingYearMonthDuration::collapseTreeInternal(DynamicContext* context, int flags) const
{
  XPath2MemoryManager* memMgr = context->getMemoryManager();

  Sequence op1 = getParamNumber(1, context);
  if(op1.isEmpty()) {
    return Sequence(memMgr);
  }

  Sequence op2 = getParamNumber(2, context);
  if (op2.isEmpty()) {
    return Sequence(memMgr);
  }

  const ATDateOrDerived* date1 = (const ATDateOrDerived*)op1.first();
  const ATDateOrDerived* date2 = (const ATDateOrDerived*)op2.first();
  
  // If one or both arguments do not have a timezone they are assigned an implicit timezone
  if (date1->hasTimezone() == false) {
    date1 = date1->addTimezone(context->getImplicitTimezone(), context);
  }
  
  if (date2->hasTimezone() == false) {
    date2 = date2->addTimezone(context->getImplicitTimezone(), context);
  }

  const ATDateTimeOrDerived* dateTime1=(const ATDateTimeOrDerived*)date1->castAs(
                                            XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
                                            XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DATETIME,
                                            context);
  const ATDateTimeOrDerived* dateTime2=(const ATDateTimeOrDerived*)date2->castAs(
                                            XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
                                            XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgDT_DATETIME,
                                            context);

  return Sequence( dateTime1->subtractDateTimeAsYearMonthDuration(dateTime2, context), memMgr);
}
















