#include "../config/pathan_config.h"
/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include <assert.h>
#include <pathan/operators/GreaterThanEqual.hpp>
#include <pathan/operators/LessThan.hpp>
#include <pathan/ATBooleanOrDerived.hpp>
#include <pathan/DynamicContext.hpp>
#include <pathan/internal/factory/DatatypeFactory.hpp>

/*static*/ const XMLCh GreaterThanEqual::name[]={ XERCES_CPP_NAMESPACE_QUALIFIER chLatin_g, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chNull };

GreaterThanEqual::GreaterThanEqual(const VectorOfDataItems &args, XPath2MemoryManager* memMgr)
  : DataItemOperator(name, args, memMgr)
{
}

/*static*/ bool GreaterThanEqual::greater_than_equal(const Item* arg1, const Item* arg2, DynamicContext* context)
{
    // define "ge" as "!lt"
    return !LessThan::less_than(arg1,arg2,context);
}

Sequence GreaterThanEqual::collapseTreeInternal(DynamicContext* context, int flags) const
{
	XPath2MemoryManager* memMgr = context->getMemoryManager();

    Sequence newArgs = getComparisonOperatorArguments(context);
    if(newArgs.getLength() < 2)
        return Sequence(memMgr);
    const Item* arg1 = newArgs.first();
    const Item* arg2 = newArgs.second();

    assert(arg1->isAtomicValue() && arg2->isAtomicValue());
    bool result=greater_than_equal(arg1,arg2,context);
    return Sequence(DatatypeFactory::POD2AT::createBoolean(memMgr, result, context), memMgr);
}

